;; Generated package description from denote.el  -*- mode: lisp-data; no-byte-compile: t; lexical-binding:t -*-
(define-package "denote" "4.2.0" "Simple notes with an efficient file-naming scheme" '((emacs "28.1")) :commit "13b2164369dd772c467fc78439bde3ec6dde965f" :authors '(("Protesilaos" . "info@protesilaos.com")) :maintainer '("Protesilaos" . "info@protesilaos.com") :url "https://github.com/protesilaos/denote")
